use chrono::{DateTime, NaiveDateTime, TimeZone, Utc};
use csv::ReaderBuilder;
use leptos::prelude::*;
use leptos_leaflet::prelude::*;
use serde::Deserialize;

pub fn to_polyline(track: &Option<GpsTrack>) -> impl IntoView {
    if let Some(track) = track {
        // Extract a vector of (latitude, longitude) tuples from the track points.
        let positions_vec: Vec<(f64, f64)> = track
            .points
            .iter()
            .map(|point| (point.latitude, point.longitude))
            .collect();

        // Return a Polyline view using the positions helper.
        view! { <Polyline positions=positions(&positions_vec) /> }.into_any()
    } else {
        view! {}.into_any()
    }
}

#[derive(Debug, Clone)]
pub struct GpsTrack {
    pub points: Vec<GpsTrackPoint>,
}

impl From<&str> for GpsTrack {
    fn from(value: &str) -> Self {
        let mut reader = ReaderBuilder::new()
            .delimiter(b',')
            .from_reader(value.as_bytes());

        Self {
            points: reader.deserialize().map(|v| v.unwrap()).collect(),
        }
    }
}

#[derive(Debug, Clone)]
pub struct GpsTrackPoint {
    pub index: u32,
    pub tag: char,
    pub timestamp: DateTime<Utc>,
    pub latitude: f64,
    pub longitude: f64,
    pub height: i32,
    pub speed: f32,
    pub heading: u32,
}

impl<'de> Deserialize<'de> for GpsTrackPoint {
    fn deserialize<D>(deserializer: D) -> Result<Self, D::Error>
    where
        D: serde::Deserializer<'de>,
    {
        #[derive(Deserialize)]
        #[serde(rename_all = "UPPERCASE")]
        struct Helper {
            index: u32,
            tag: char,
            date: String, // e.g., "250301" (ddMMyy)
            time: String, // e.g., "101102" (HHMMSS)
            #[serde(rename = "LATITUDE N/S")]
            latitude: String, // e.g., "51.9787868N"
            #[serde(rename = "LONGITUDE E/W")]
            longitude: String, // e.g., "9.2337406E"
            height: i32,
            speed: f32,
            heading: u32,
        }

        let helper = Helper::deserialize(deserializer)?;

        // Combine DATE and TIME into one string, then parse.
        let datetime_str = format!("{} {}", helper.date, helper.time);
        let naive = NaiveDateTime::parse_from_str(&datetime_str, "%y%m%d %H%M%S")
            .map_err(serde::de::Error::custom)?;
        // Use the TimeZone trait's method from_utc_datetime.
        let timestamp = Utc.from_utc_datetime(&naive);

        fn parse_coord<E>(coord: &str) -> Result<f64, E>
        where
            E: serde::de::Error,
        {
            if coord.len() < 2 {
                return Err(serde::de::Error::custom("Coordinate string too short"));
            }
            let (number_part, direction) = coord.split_at(coord.len() - 1);
            let value: f64 = number_part.parse().map_err(serde::de::Error::custom)?;
            match direction {
                "N" | "E" => Ok(value),
                "S" | "W" => Ok(-value),
                _ => Err(serde::de::Error::custom("Invalid coordinate direction")),
            }
        }

        let latitude = parse_coord::<D::Error>(&helper.latitude)?;
        let longitude = parse_coord::<D::Error>(&helper.longitude)?;

        Ok(GpsTrackPoint {
            index: helper.index,
            tag: helper.tag,
            timestamp,
            latitude,
            longitude,
            height: helper.height,
            speed: helper.speed,
            heading: helper.heading,
        })
    }
}
