use crate::gps::gps_track::to_polyline;
use crate::{components::file_select::FileSelect, gps::gps_track::GpsTrack};
use leptos::logging::log;
use leptos::prelude::*;
use leptos::wasm_bindgen::JsCast;
use leptos_leaflet::leaflet::*;
use leptos_leaflet::prelude::*;
use wasm_bindgen::prelude::Closure;
use web_sys::{File, FileReader};

/// Default Home Page
#[component]
pub fn Home() -> impl IntoView {
    let track = RwSignal::new(None);
    let map: RwSignal<Option<leptos_leaflet::leaflet::Map>, _> = RwSignal::new_local(None);

    // Function to handle file selection
    let on_file_select = move |file: File| {
        let reader = FileReader::new().expect("Failed to create FileReader");

        // Clone the signal to move into the closure
        let track_clone = track.clone();

        // Define the callback to handle the 'load' event
        let onload = Closure::wrap(Box::new(move |event: web_sys::Event| {
            let reader = event.target().unwrap().dyn_into::<FileReader>().unwrap();
            if let Ok(result) = reader.result() {
                if let Some(text) = result.as_string() {
                    log!("File content: {}", text);
                    track_clone.set(Some(GpsTrack::from(text.as_str())));
                }
            }
        }) as Box<dyn FnMut(_)>);

        // Set the callback for the 'load' event
        reader.set_onload(Some(onload.as_ref().unchecked_ref()));

        // Read the file as text
        reader.read_as_text(&file).expect("Failed to read file");

        // Keep the closure alive
        onload.forget();
    };

    Effect::watch(
        move || track.get(),
        move |track, _, _| {
            map.get().as_ref().unwrap().set_view(
                &LatLng::new(
                    track.as_ref().unwrap().points[0].latitude,
                    track.as_ref().unwrap().points[0].longitude,
                ),
                35.0,
            )
        },
        false,
    );

    view! {
        <ErrorBoundary fallback=|errors| {
            view! {
                <h1>"Uh oh! Something went wrong!"</h1>

                <p>"Errors: "</p>
                // Render a list of errors as strings - good for development purposes
                <ul>
                    {move || {
                        errors
                            .get()
                            .into_iter()
                            .map(|(_, e)| view! { <li>{e.to_string()}</li> })
                            .collect_view()
                    }}

                </ul>
            }
        }>

            <div class="flex flex-col items-center p-5 gap-4 h-full w-full flex-grow">
                <h1 class="text-3xl text-bold">"Import GPS Trace"</h1>

                <FileSelect
                    class="p-2 rounded-md bg-gray-300 hover:bg-gray-400 select-none"
                    on_file_select=on_file_select
                >
                    "Select Trace File"
                </FileSelect>
                <MapContainer
                    center=Position::new(0.0, 0.0)
                    zoom=3.0
                    class="flex-grow w-full"
                    map=map.write_only()
                    set_view=true
                >
                    <TileLayer
                        url="https://tile.openstreetmap.org/{z}/{x}/{y}.png"
                        attribution="&copy; <a href=\"https://www.openstreetmap.org/copyright\">OpenStreetMap</a> contributors"
                    />
                    {{ move || to_polyline(&track.get()) }}
                </MapContainer>
            </div>
        </ErrorBoundary>
    }
}
